from dataclasses import dataclass


@dataclass
class DataBase:
    PUBMED = "pubmed"
    PROTEIN = "protein"
    NUCCORE = "nuccore"
    IPG = "ipg"
    NUCLEOTIDE = "nucleotide"
    STRUCTURE = "structure"
    GENOME = "genome"
    ANNOTINFO = "annotinfo"
    ASSEMBLY = "assembly"
    BIOPROJECT = "bioproject"
    BIOSAMPLE = "biosample"
    BLASTDBINFO = "blastdbinfo"
    BOOKS = "books"
    CDD = "cdd"
    CLINVAR = "clinvar"
    GAP = "gap"
    GAPPLUS = "gapplus"
    GRASP = "grasp"
    DBVAR = "dbvar"
    GENE = "gene"
    GDS = "gds"
    GEOPROFILES = "geoprofiles"
    HOMOLOGENE = "homologene"
    MEDGEN = "medgen"
    MESH = "mesh"
    NLMCATALOG = "nlmcatalog"
    OMIM = "omim"
    ORGTRACK = "orgtrack"
    PMC = "pmc"
    POPSET = "popset"
    PROTEINCLUSTERS = "proteinclusters"
    PCASSAY = "pcassay"
    PROTFAM = "protfam"
    PCCOMPOUND = "pccompound"
    PCSUBSTANCE = "pcsubstance"
    SEQANNOT = "seqannot"
    SNP = "snp"
    SRA = "sra"
    TAXONOMY = "taxonomy"
    BIOCOLLECTIONS = "biocollections"
    GTR = "gtr"
